#!/usr/bin/python

import math
import re
import lxml.etree
import svg.path
import sys
import math

doc=lxml.etree.parse(sys.argv[1])

def apply(matrix, x, y):
    if matrix is None:
        return (x, y)

    (a, b, c, d, e, f)=matrix
    return (a*x + c*y + e,
            b*x + d*y + f)

def applyC(matrix, c):
    (x, y)=apply(matrix, c.real, c.imag)
    return complex(x, y)

def mult(m1, m2):
    (a1, b1, c1, d1, e1, f1) = m1
    (a2, b2, c2, d2, e2, f2) = m2
    return [a1*a2+c1*b2,
            b1*a2+d1*b2,
            a1*c2+c1*d2,
            b1*c2+d1*d2,
            a1*e2+c1*f2+e1,
            b1*e2+d1*f2+f1]

clipPathToElt={}
clipPathBeenUsed={}

def handle_transform(elt, attribName='transform'):
    if attribName in elt.attrib:
        operator=elt.attrib[attribName].split('(')[0]
        operandStr=elt.attrib[attribName].split('(')[1].split(')')[0]
        if ',' in operandStr:
            operands=map(float,operandStr.split(','))
        else:
            operands=map(float,operandStr.split(' '))
        del elt.attrib[attribName]

        if operator=='matrix':
            assert len(operands)==6
            return operands

        if operator=='translate':
            assert len(operands) in [1,2]
            if len(operands)==1:
                operands.append(0)
            return [1,0,0,1]+operands

        if operator=='scale':
            assert len(operands) in [1,2]
            if len(operands)==1:
                x=operands[0]
                y=x
            else:
                (x, y)=operands
            return [x, 0, 0, y, 0, 0]

        if operator=='rotate':
            assert len(operands)>=1
            assert len(operands)<=3
            if len(operands)<2:
                operands.append(0)
            if len(operands)<3:
                operands.append(0)
            (th, x, y)=operands
            th=th*math.pi/180
            sys.stderr.write("th: %f\n(x, y): (%f, %f)\n" % (th, x, y))
            shift=[1,0,0,1,-x,-y]
            rotation=[math.cos(th), math.sin(th), -math.sin(th), math.cos(th),
                      0, 0]
            shiftBack=[1,0,0,1,x,y]
            res=mult(mult(shiftBack, rotation), shift)
            sys.stderr.write("shift: %s\n" % str(shift))
            sys.stderr.write("rotation: %s\n" % str(rotation))
            sys.stderr.write("shiftBack: %s\n" % str(shiftBack))
            sys.stderr.write("res: %s\n" % str(res))
            return res
            
        # These aren't used, either, but they're easy.
        if operator=='skewX':
            assert len(operands)==1
            return [1, math.tan(operands[0]), 0, 0, 1, 0]

        if operator=='skewY':
            assert len(operands)==1
            return [1, 0, 0, math.tan(operands[0]), 1, 0]

        # Some transform not in SVG 1.1...as far as I know.
        assert False

def begin_path(elt, matrix):
    path=svg.path.parse_path(elt.attrib['d'])
    for segment in path:
        if isinstance(segment, svg.path.CubicBezier):
            segment.start=applyC(matrix, segment.start)
            segment.control1=applyC(matrix, segment.control1)
            segment.control2=applyC(matrix, segment.control2)
            segment.end=applyC(matrix, segment.end)
        else:
            if isinstance(segment, svg.path.Line):
                segment.start=applyC(matrix, segment.start)
                segment.end=applyC(matrix, segment.end)
            else:
                if isinstance(segment, svg.path.Arc):
                    segment.start=applyC(matrix, segment.start)
                    segment.end=applyC(matrix, segment.end)
                else:
                    if isinstance(segment, svg.path.QuadraticBezier):
                        segment.start=applyC(matrix, segment.start)
                        segment.control=applyC(matrix, segment.control)
                        segment.end=applyC(matrix, segment.end)
    elt.attrib['d']=path.d(True)

# NB: this code assumes that a clipPath gets used no more than once.
def begin_clipPath(elt):
    global curClipPathID, clipPathToContents, clipPathBeenUsed
    curClipPathID=elt.attrib['id']
    assert curClipPathID not in clipPathToContents
    clipPathToContents[curClipPathID]=[]
    clipPathBeenUsed[curClipPathID]=False

def end_clipPath(elt):
    global curClipPathID
    assert curClipPathID==elt.attrib['id']
    curClipPathID=None

def begin_rect(elt, matrix):
    x=float(elt.attrib['x'])
    y=float(elt.attrib['y'])
    width=float(elt.attrib['width'])
    height=float(elt.attrib['height'])
    x2=x+width
    y2=y+height
    (x, y)=apply(matrix, x, y)
    (x2, y2)=apply(matrix, x2, y2)
    width=x2-x
    height=y2-y

    if width < 0:
        (x, x2) = (x2, x)
        width = -width

    if height < 0:
        (y, y2) = (y2, y)
        height = -height

    elt.attrib['x']=str(x)
    elt.attrib['y']=str(y)
    elt.attrib['width']=str(width)
    elt.attrib['height']=str(height)

def begin_ellipse(elt, matrix):
    cx=float(elt.attrib['cx'])
    cy=float(elt.attrib['cy'])
    rx=float(elt.attrib['rx'])
    ry=float(elt.attrib['ry'])
    (cx, cy)=apply(matrix, cx, cy)
    (zx, zy)=apply(matrix, 0, 0)
    (rx, ry)=apply(matrix, rx, ry)
    rx-=zx
    ry-=zy
    elt.attrib['cx']=str(cx)
    elt.attrib['cy']=str(cy)
    elt.attrib['rx']=str(abs(rx))
    elt.attrib['ry']=str(abs(ry))
    
def begin_circle(elt, matrix):
    cx=float(elt.attrib['cx'])
    cy=float(elt.attrib['cy'])
    rx=float(elt.attrib['r'])
    ry=float(elt.attrib['r'])
    (cx, cy)=apply(matrix, cx, cy)
    (zx, zy)=apply(matrix, 0, 0)
    (rx, ry)=apply(matrix, rx, ry)
    rx-=zx
    ry-=zy
    tagParts = elt.tag.split('}')
    tagParts[1] = 'ellipse'
    elt.tag = '}'.join(tagParts)
    del elt.attrib['r']
    elt.attrib['cx']=str(cx)
    elt.attrib['cy']=str(cy)
    elt.attrib['rx']=str(abs(rx))
    elt.attrib['ry']=str(abs(ry))

linearGradientTable={}

def begin_linearGradient(elt):
    if (('gradientUnits' in elt.attrib) and
        (elt.attrib['gradientUnits']=='userSpaceOnUse')):
        linearGradientTable[elt.attrib['id']]=elt

def mungLinearGradient(elt, matrix):
    transform=handle_transform(elt, 'gradientTransform')
    if transform:
        matrix=mult(matrix, transform)
    if 'x1' in elt.attrib:
        x1=float(elt.attrib['x1'])
        y1=float(elt.attrib['y1'])
        x2=float(elt.attrib['x2'])
        y2=float(elt.attrib['y2'])
        (x1, y1)=apply(matrix, x1, y1)
        (x2, y2)=apply(matrix, x2, y2)
        elt.attrib['x1']=str(x1)
        elt.attrib['y1']=str(y1)
        elt.attrib['x2']=str(x2)
        elt.attrib['y2']=str(y2)

urlInfix='fill:url(#'

def findAndMungLinearGradient(style, matrix):
    gradientId=style.split(urlInfix)[1].split(')')[0]
    if gradientId in linearGradientTable:
        elt=linearGradientTable[gradientId]
        del linearGradientTable[gradientId]
        mungLinearGradient(elt, matrix)

def get_tag(elt):
    return elt.tag.split('}')[1]

def apply_cur_matrix(elt, matrix, suppressClipPath=False):
    tag=get_tag(elt)
    if tag=='path':
        begin_path(elt, matrix)
    else:
        if tag=='rect':
            begin_rect(elt, matrix)
        else:
            if tag=='ellipse':
                begin_ellipse(elt, matrix)
            else:
                if tag=='circle':
                    begin_circle(elt, matrix)
                else:
                    if tag=='linearGradient':
                        begin_linearGradient(elt)
    if 'style' in elt.attrib and urlInfix in elt.attrib['style']:
        findAndMungLinearGradient(elt.attrib['style'], matrix)

def descend(elt, matrix, inClipPath):
    for d in elt.iterchildren():
        tag=get_tag(d)
        if tag=='clipPath':
            id=d.attrib['id']
            assert id not in clipPathToElt
            clipPathToElt[id]=d
            clipPathBeenUsed[id]=False
            continue
        oldMatrix = matrix
        curMatrix = handle_transform(d)
        if curMatrix:
            # I'm still not sure I've got these in the right order.
            matrix=mult(matrix, curMatrix)
        if 'clip-path' in d.attrib:
            assert not inClipPath
            url=d.attrib['clip-path']
            prefix='url(#'
            assert url.startswith(prefix)
            assert url[-1]==')'
            id=url[len(prefix):-1]
            assert id in clipPathToElt
            assert not clipPathBeenUsed[id]
            clipPathBeenUsed[id]=True
            descend(clipPathToElt[id], matrix, True)

        apply_cur_matrix(d, matrix)
        descend(d, matrix, inClipPath)
        matrix=oldMatrix
        if d.tag.split('}')[1]=='clipPath':
            end_clipPath(d)

descend(doc.getroot(), [1, 0, 0, 1, 0, -800], False)

if 'viewBox' in doc.getroot().attrib:
    del doc.getroot().attrib['viewBox']

doc.write(sys.stdout)
sys.stdout.write("\n")
